import ctypes

class C_Response(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p))
    ]

class C_Node(ctypes.Structure):
    _fields_ = [
        ('ai_model', ctypes.c_char * 64),
        ('hash', ctypes.c_char * 64),
        ('normal', ctypes.c_char * 2),
        ('vdim', ctypes.c_int),
        ('vp', ctypes.c_double * 2048)
    ]    

class C_PutVectorRS(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p)),
        ('hash', ctypes.POINTER(ctypes.c_char_p))  
    ]

class C_GetVectorRS(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p)),
        ('node', C_Node)
    ]

class C_QueryVectorRS(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.c_char * 18),
        ('ai_model', ctypes.c_char * 64),
        ('normal', ctypes.c_char * 2),
        ('hash', ctypes.c_char * 64),
        ('vdim', ctypes.c_int),
        ('distance', ctypes.c_double)
    ]  

class C_QueryVectorRSWrapper(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p)),
        ('queryCount', ctypes.c_int),
        ('faultCount', ctypes.c_int),
        ('queryVectorRS', ctypes.POINTER(C_QueryVectorRS)),
        ('faultVectorRS', ctypes.POINTER(C_QueryVectorRS))
    ] 

class C_CountRS(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p)),
        ('count', ctypes.c_int)
    ]

class C_CollectionListRS(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p)),
        ('collections', ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p)))
    ]

class C_VectorListRS(ctypes.Structure):
    _fields_ = [
        ('errCode', ctypes.c_int),
        ('errMsg', ctypes.POINTER(ctypes.c_char_p)),
        ('vectors', ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p)))
    ]

class C_QueryOptions(ctypes.Structure):
    _fields_ = [
        ('vectorDistanceMethod', ctypes.c_int),
        ('queryLimit', ctypes.c_int),
        ('queryLogicalOP', ctypes.c_int),
        ('queryValue', ctypes.c_double),
        ('includeFault', ctypes.c_bool),
        ('pValue', ctypes.c_double),
        ('doNormal', ctypes.c_bool),
        ('order', ctypes.c_bool),
    ]



import ctypes
from victo.c_ds import *

class Response:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')

class Node:
    def __init__(self, obj):
        self.ai_model = obj.node.ai_model if obj.errCode == 0 else None # ctypes.string_at(obj.node.ai_model).decode('utf-8') if obj.errCode == 0 else None
        self.hash = obj.node.hash if obj.errCode == 0 else None # ctypes.string_at(obj.node.hash).decode('utf-8') if obj.errCode == 0 else None
        self.normal = obj.node.normal if obj.errCode == 0 else None # ctypes.string_at(obj.node.normal).decode('utf-8') if obj.errCode == 0 else None
        self.vdim = obj.node.vdim if obj.errCode == 0 else 0
        self.vp = [obj.node.vp[i] for i in range(obj.node.vdim)] if obj.errCode == 0 else []

class PutVectorRS:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')
        self.hash = ctypes.string_at(obj.hash).decode('utf-8')

class GetVectorRS:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')
        self.node = Node(obj)

class QueryVectorRS:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg =  obj.errMsg # ctypes.string_at(obj.errMsg).decode('utf-8') if obj.errCode == 0 else None
        self.ai_model = obj.ai_model # ctypes.string_at(obj.ai_model).decode('utf-8') if obj.errCode == 0 else None
        self.normal = obj.normal  # ctypes.string_at(obj.normal).decode('utf-8') if obj.errCode == 0 else None
        self.hash = obj.hash  # ctypes.string_at(obj.hash).decode('utf-8') if obj.errCode == 0 else None
        self.vdim = obj.vdim 
        self.distance = obj.distance if obj.errCode == 0 else None

class QueryVectorRSWrapper:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')
        self.queryCount = obj.queryCount
        self.faultCount = obj.faultCount
        self.queryVectorRS = [QueryVectorRS(obj.queryVectorRS[i]) for i in range(obj.queryCount)] 
        self.faultVectorRS = [QueryVectorRS(obj.faultVectorRS[i]) for i in range(obj.faultCount)]

class CountRS:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')
        self.count = obj.count

class CollectionListRS:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')
        self.collections = []
        collections_pointer = obj.collections
        i = 0
        while collections_pointer[i]:
            self.collections.append(ctypes.string_at(collections_pointer[i]).decode('utf-8'))
            i += 1

class VectorListRS:
    def __init__(self, obj):
        self.errCode = obj.errCode
        self.errMsg = ctypes.string_at(obj.errMsg).decode('utf-8')
        self.vectors = []
        vectors_pointer = obj.vectors
        i = 0
        while vectors_pointer[i]:
            self.vectors.append(ctypes.string_at(vectors_pointer[i]).decode('utf-8'))
            i += 1

class QueryOptions(ctypes.Structure):
    def __init__(self, obj):
        self.vectorDistanceMethod = obj.vectorDistanceMethod
        self.queryLimit = obj.queryLimit
        self.queryLogicalOP = obj.queryLogicalOP
        self.queryValue = obj.queryValue
        self.includeFault = obj.includeFault
        self.pValue = obj.pValue
        self.doNormal = obj.doNormal
        self.order = obj.order





from victo.py_ds import *
from victo.c_ds import *
import ctypes
import os
import platform



class CProxy:    
    def __init__(self):
        system = platform.system()
    
        if system == 'Darwin':
            libName = 'libvicto.dylib'
            sysName = 'macos'
        elif system == 'Linux':
            libName = 'libvicto.so'
            sysName = 'linux'
        else:
            raise RuntimeError("Unsupported Platform")

        libPath = os.path.join(os.path.dirname(__file__),'lib', sysName, libName);
        self.c_lib = ctypes.cdll.LoadLibrary(libPath)


    def newCollection(self, location):
        newCollectionFunc = self.c_lib.newCollectionSL
        newCollectionFunc.restype = C_Response
        newCollectionFunc.argtypes = [ctypes.c_char_p]
        rs = newCollectionFunc(location.encode('utf-8'))
        return Response(rs)

    def deleteCollection(self, location):
        deleteCollectionFunc = self.c_lib.deleteCollectionSL
        deleteCollectionFunc.restype = C_Response
        deleteCollectionFunc.argtypes = [ctypes.c_char_p]
        rs = deleteCollectionFunc(location.encode('utf-8'))
        return Response(rs)
    
    def collectionCount(self, location):
        collectionCountFunc = self.c_lib.collectionCountSL
        collectionCountFunc.restype = C_CountRS
        collectionCountFunc.argtypes = [ctypes.c_char_p]
        rs = collectionCountFunc(location.encode('utf-8'))
        return CountRS(rs)
    
    def collectionList(self, location):
        collectionListFunc = self.c_lib.collectionListSL
        collectionListFunc.restype = C_CollectionListRS
        collectionListFunc.argtypes = [ctypes.c_char_p]
        rs = collectionListFunc(location.encode('utf-8'))
        return CollectionListRS(rs)

    def putVector(self, location, ai_model, hash, vdim, vp, is_normal, overwrite):
        c_vp = (ctypes.c_double * vdim) (*vp)
        putVectorFunc = self.c_lib.putVectorSL       
        putVectorFunc.restype = C_PutVectorRS
        putVectorFunc.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_int, ctypes.POINTER(ctypes.c_double), ctypes.c_bool, ctypes.c_bool]
        rs = putVectorFunc(location.encode('utf-8'), ai_model.encode('utf-8'), hash.encode('utf-8'), vdim,c_vp,is_normal, overwrite);
        return PutVectorRS(rs)       

    def getVector(self, location, hash):
        getVectorFunc = self.c_lib.getVectorSL
        getVectorFunc.restype = C_GetVectorRS
        getVectorFunc.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
        rs = getVectorFunc(location.encode('utf-8'), hash.encode('utf-8'))
        return GetVectorRS(rs)

    def queryVector(self, location, ai_model, vdim, vp, vector_distance_method, query_limit, logical_op, query_value, p_value, do_normal, include_fault, order):
        c_vp = (ctypes.c_double * vdim) (*vp)
        queryOptions = C_QueryOptions()
        queryOptions.vectorDistanceMethod = vector_distance_method
        queryOptions.queryLimit = query_limit
        queryOptions.queryLogicalOP = logical_op
        queryOptions.queryValue = query_value
        queryOptions.includeFault = include_fault
        queryOptions.pValue = p_value
        queryOptions.doNormal = do_normal
        queryOptions.order = order

        queryVectorFunc = self.c_lib.queryVectorSL
        queryVectorFunc.restype = C_QueryVectorRSWrapper
        queryVectorFunc.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_int, ctypes.POINTER(ctypes.c_double), C_QueryOptions]
        rs = queryVectorFunc(location.encode('utf-8'), ai_model.encode('utf-8'), vdim, c_vp, queryOptions)
        return QueryVectorRSWrapper(rs)

    def deleteVector(self, location, hash):
        deleteVectorFunc = self.c_lib.deleteVectorSL
        deleteVectorFunc.restype = C_Response
        deleteVectorFunc.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
        rs = deleteVectorFunc(location.encode('utf-8'), hash.encode('utf-8'))
        return Response(rs)
    
    def vectorCount(self, location):
        vectorCountFunc = self.c_lib.vectorCountSL
        vectorCountFunc.restype = C_CountRS
        vectorCountFunc.argtypes = [ctypes.c_char_p]
        rs = vectorCountFunc(location.encode('utf-8'))
        return CountRS(rs)
    
    def vectorList(self, location):
        vectorListFunc = self.c_lib.vectorListSL
        vectorListFunc.restype = C_VectorListRS
        vectorListFunc.argtypes = [ctypes.c_char_p]
        rs = vectorListFunc(location.encode('utf-8'))
        return VectorListRS(rs)




import os
from victo.cproxy import CProxy

cProxy = CProxy()

def newCollection(db, collection):
    return cProxy.newCollection(os.path.join(db, collection))


def deleteCollection(db, collection):
    return cProxy.deleteCollection(os.path.join(db, collection))

    
def collectionCount(db):
    return cProxy.collectionCount(db)

    
def collectionList(db):
    return cProxy.collectionList(db)


def putVector(db, collection, ai_model, hash, vdim, vp, is_normal=False, overwrite=False):
    return cProxy.putVector(os.path.join(db, collection), ai_model, hash, vdim, vp, is_normal, overwrite)
    

def getVector(db, collection, hash):
    return cProxy.getVector(os.path.join(db, collection), hash)

def queryVector(db, collection, ai_model, vdim, vp, vector_distance_method=0, query_limit=-99, logical_op=0, k_value=0, p_value=0, do_normal=False, include_fault=False, order=True):
    return cProxy.queryVector(os.path.join(db, collection), ai_model, vdim, vp, vector_distance_method, query_limit, logical_op, k_value, p_value, do_normal, include_fault, order)


def deleteVector(db, collection, hash):
    return cProxy.deleteVector(os.path.join(db, collection), hash)

    
def vectorCount(db, collection):
    return cProxy.vectorCount(os.path.join(db, collection))

    
def vectorList(db, collection):
    return cProxy.vectorList(os.path.join(db, collection))


